# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include <stdlib.h>

   # include "color_theme.h"
   # include "cursor.h"
   # include "cwrite.h"
   # include "dataline.h"
   # include "datasystem.h"
   # include "header_line.h"
   # include "mycurses.h"
   # include "status_line.h"
   # include "string.h"
# endif
# if GLOBAL
   int screen_cols = 80;
   int screen_lines = 25;
   int screen_left = 3;
   int screen_right;
   int screen_top = 3;
   int screen_bottom;

   char * _empty_line;
   int cursor_y, cursor_x;
   int status_register = 0;
# endif
# if PROTO
   void screen_line_redraw(Datachain * _datachain, int cursor_y);
   void screen_redraw();
# endif

void screen_init()
   {    // -- ! init screen_cols --
     screen_right = screen_cols - 1;
     screen_bottom = screen_lines - 1;
     //dataline_extend_size = screen_cols - screen_left - 1;
     dataline_extend_size = screen_cols - screen_left - 1;
     dataline_start_size = dataline_extend_size;
     _empty_line = (char *) malloc(screen_cols);
     _memset(_empty_line, ' ', screen_cols);

     return;
   }
void screen_cursor()
  { cursor_y = HEADER_LINE;
    cursor_x = 0;

       // -- sets coords when no data system --
    _cursor_y = & cursor_y;
    _cursor_x = & cursor_x;

      // -- avoid seg on usage --
    _status_register = & status_register;
     return;
  }

void screen_clear()
   { curses_clear();

     header_line_refresh();
     status_line_refresh();

     return;
   }

void screen_line_redraw(Datachain * _datachain, int cursor_y)
   { Datachain * _datachain_next = _datachain->_next;
     bool tree_view = 0;

     if ( _datachain_next )
        { if ( _datachain_next->invisible )
             { tree_view = 1;
               if ( * _status_register & DETACH_STATE && _datachain == * __datachain_detach_end )
                  { tree_view = 0;
                  }
             }
        }
     if ( ! tree_view )
        { curses_mvaddnstr(cursor_y, 0, "   ", 3);
        }
     else
        { curses_mvaddnstr(cursor_y, 0, " + ", 3);
          curses_mvchgat(cursor_y, 1, 1, A_NORMAL, block_color_pair, 0);
        }
     char * _dataline = _datachain->_line;
     int delta = * _dataline_offset - * _cursor_x + screen_left;
     curses_mvaddnstr(cursor_y, screen_left, _dataline + delta, screen_cols - screen_left);
     cursor_move(* _cursor_y, * _cursor_x);

     return;
   }

void screen_line_empty(int cursor_y)
   { curses_mvaddnstr(cursor_y, 0, _empty_line, screen_cols);
     cursor_move(* _cursor_y, * _cursor_x);
     return;
   }

void screen_redraw()
   { if ( ! datasystem_count )
        { screen_clear();
          return;
        }
     Datachain * _datachain = * __datachain;
     int cursor_y = * _cursor_y;
            
     while ( cursor_y <= screen_bottom )
        {  if ( _datachain )
              { if ( ! _datachain->invisible )
                   { screen_line_redraw(_datachain, cursor_y++);
                   }
                _datachain = _datachain->_next;
              }
           else
              { screen_line_empty(cursor_y++);
              }
        }
     _datachain = (* __datachain)->_prev;
      cursor_y = (* _cursor_y) - 1;
      while ( cursor_y >= screen_top )
        {  if ( _datachain )
              { if ( ! _datachain->invisible )
                   { screen_line_redraw(_datachain, cursor_y--);
                   }
                _datachain = _datachain->_prev;
              }
           else
              { screen_line_empty(cursor_y--);
              }
        }
      return;
   }
