# include "std.h"
# if LICENCE
   This file is part of cwrite.
   Copyright 2007, Desislav Georgiev.
 
   cwrite is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
 
   cwrite is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with cwrite.  If not, see <http://www.gnu.org/licenses/>.
# endif
# if HEADER
   # include "cwrite.h"
   # include "datasystem.h"
   # include "screen.h"
# endif
# if PROTO
   int visible_lines_down(Datachain * _datachain);
   int visible_lines_up(Datachain* _datachain);
   bool seek_visible_line_down();
   bool seek_visible_line_up();
# endif

bool scroll_left()
   { if ( * _dataline_offset == * _cursor_x - screen_left )
        { return 0;
        }
     (* _dataline_offset)--;
     screen_redraw();

     return 1;
   }
bool scroll_right()
   { int offset = _datasystem->dataline_max_size + 1 - screen_cols + * _cursor_x;
     if ( * _dataline_offset == offset )
        { return 0;
        }
     (* _dataline_offset)++;
     screen_redraw();

     return 1;
   }
bool scroll_home()
   { int offset = * _cursor_x - screen_left;
     if ( * _dataline_offset == offset )
        { return 0;
        }
     * _dataline_offset = offset;
     screen_redraw();

     return 1;
   }
bool scroll_end()
   { int offset = _datasystem->dataline_max_size + 1 - screen_cols + * _cursor_x;
     if ( * _dataline_offset == offset )
        { return 0;
        }
     * _dataline_offset = offset;
     screen_redraw();

     return 1;
   }
bool scroll_down()
   { int visible_lines = visible_lines_down(* __datachain);
     if ( ! visible_lines )
        { return 0;
        }
     if ( visible_lines <= screen_bottom - * _cursor_y + 1)
        { return 0;
        }
     seek_visible_line_down();
     screen_redraw();

     return 1;
   }
bool scroll_up()
   { int visible_lines = visible_lines_up(* __datachain);
     if ( ! visible_lines )
        { return 0;
        }
     if ( visible_lines == * _cursor_y - screen_top + 1 )
        { return 0;
        }
     seek_visible_line_up();
     screen_redraw();

     return 1;
   }
void scroll_npage()
   { for ( int i = 0; i < screen_lines - screen_top -1; i++ )
        { scroll_down();
        }
     return;
   }
void scroll_ppage()
   { for ( int i = 0; i < screen_lines - screen_top - 1; i++ )
        { scroll_up();
        }
     return;
   }

int visible_lines_down(Datachain * _datachain)
   { int visible_lines = 0;

     while ( _datachain )
        { if ( ! _datachain->invisible )
             { visible_lines++;
             }
          _datachain = _datachain->_next;
        }
     return visible_lines;
   }
int visible_lines_up(Datachain * _datachain)
   { int visible_lines = 0;

     while ( _datachain )
        { if ( ! _datachain->invisible )
             { visible_lines++;
             }
          _datachain = _datachain->_prev;
        }
     return visible_lines;
   }

bool seek_visible_line_down()
   { Datachain * _datachain = (* __datachain)->_next;

     while ( _datachain )
        { if ( ! _datachain->invisible )
             { * __datachain = _datachain;
               return 1;
             }
          _datachain = _datachain->_next;
        }
     return 0;
   }
bool seek_visible_line_up()
   { Datachain * _datachain = (* __datachain)->_prev;

     while ( _datachain )
        { if ( ! _datachain->invisible )
            { * __datachain = _datachain;
              return 1;
            }
          _datachain = _datachain->_prev;
        }
     return 0;
   }

Datachain * _next_visible_line_down(Datachain * _datachain_begin)
   { Datachain * _datachain = _datachain_begin;

     do
       { _datachain = _datachain->_next;
         if ( _datachain )
            { if ( ! _datachain->invisible )
                { return _datachain;
                }
            }
        }
     while ( _datachain );
     return 0;
   }
Datachain * _next_visible_line_up(Datachain * _datachain_begin)
   { Datachain * _datachain = _datachain_begin;

     do
       { _datachain = _datachain->_prev;
         if ( _datachain )
            { if ( ! _datachain->invisible )
                 { return _datachain;
                 }
            }
       }
     while ( _datachain );
     return 0;
   }
